/******************************************************************************
 *            echecs_init.c                                                   *
 *                                                                            *
 *  Copyright  2006  gerald.dumas                                             *
 *  linechec@laposte.net                                                      *
 ******************************************************************************
 *                                                                            *
 *  This program is free software; you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  any later version.                                                        *
 *                                                                            *
 *  This program is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with this program; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 ******************************************************************************/

/* librairie initialisation du jeu d'echec */
#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif
#include "global.h"

#include "echecs_init.h"

// initialisation d'une pièce avec sa position sur l'échiquier
type_piece
initialisation_piece (gint pos_table, gchar piece, gint couleur,
		      gint position, gint ponderation)
{
  type_piece pieces;

  pieces.type = piece;
  pieces.couleur = couleur;
  pieces.place = position;
  pieces.deplacement_initial = TRUE;
	pieces.nbre_cases_jouables=0;
  echiquier[position].pos_table_piece = pos_table;
  pieces.ponderation = ponderation;

  return pieces;
}

// Fonction de chargement ou de création du fichier de configuration
gboolean configuration_soft (int argc, char **argv)
{
  gchar *fichier_config;
  gchar *chemin;
  gchar *chemin_soft=NULL;
  GError *error = NULL;
  GOptionContext *context;

  // Création d'un fichier de configuration dans le répertoire personnel de
  // l'utilisateur s'il n'existe pas.
	if (strstr(argv[0],"./"))
		chemin_soft = g_get_current_dir();
	else
		chemin_soft= g_strdup("/usr/bin");

  fichier_config = g_strdup_printf ("%s/.linechec.conf", g_get_home_dir ());
  key_file_config = g_key_file_new ();
  if (!g_key_file_load_from_file (key_file_config, fichier_config,
				  G_KEY_FILE_KEEP_COMMENTS, NULL))
	{
		// Le fichier de config n'existe pas. Il est créé trés allégé.
		// Il faut que l'utilisateur modifie les préférences pour l'agrémenter.
		g_key_file_set_string (key_file_config, "COMMON", "name", "linechec");
		// DEBUG by xcfaudio: VERSION from config.h
		// g_key_file_set_string (key_file_config, "COMMON", "version", "v0.4");
		g_key_file_set_string (key_file_config, "COMMON", "version", VERSION);
		g_key_file_set_string (key_file_config, "COMMON", "url",
				 "http://linechec.sourceforge.net");
		g_key_file_set_comment (key_file_config, "COMMON", NULL,
					" Linechec configuration file.\n Don't edit directly.\n", NULL);
		g_key_file_set_string (key_file_config, "COMMON", "author",
				 "Gerald Dumas (linechec@laposte.net)");
		g_key_file_set_string (key_file_config, "COMMON", "contacts",
				 "linechec@laposte.net");

		g_key_file_set_string (key_file_config, "DIRECTORY", "install",
				 chemin_soft);
		g_key_file_set_comment (key_file_config, "DIRECTORY", "install",
					" install application directory.", NULL);
		g_key_file_set_string (key_file_config, "DIRECTORY", "chess_pixmaps",
				 "gnome-chess");
		g_key_file_set_comment (key_file_config, "DIRECTORY", "chess_pixmaps",
					" Chess pixmaps directory.", NULL);

		chemin = g_key_file_to_data (key_file_config, NULL, NULL);
		if (!g_file_set_contents (fichier_config, chemin, -1, &error))
		{
			if (error->message)
				printf ("%s\n", error->message);
			gtk_main_quit ();
		}
		g_free (chemin);
	}
  else
	{
		// Le fichier de config existe. On vérifie si ce fichier n'est pas une
		// version antérieur à 0.3. Si tel est le cas, alors on le supprime et on
		// en crée un nouveau.
		if (!g_key_file_get_string (key_file_config, "COMMON", "version", NULL))
		{
			//Suppression de l'ancien fichier de configuration
			chemin = g_strdup_printf ("rm %s", fichier_config);
			g_spawn_command_line_sync (chemin, NULL, NULL, NULL, NULL);
			g_free (chemin);
			// Création d'un nouveau fichier de configuration.
			configuration_soft (argc, argv);
		}
		// DEBUG by xcfaudio: VERSION from config.h
		else //La version correspond-elle à la nouvelle version= "v0.4" ?
		// if (strcmp(g_key_file_get_string (key_file_config, "COMMON", "version", NULL),"v0.4")!=0)
		if (strcmp(g_key_file_get_string (key_file_config, "COMMON", "version", NULL),VERSION)!=0)
		{
			// g_key_file_set_string (key_file_config, "COMMON", "version", "v0.4");
			g_key_file_set_string (key_file_config, "COMMON", "version", VERSION);
			chemin = g_key_file_to_data (key_file_config, NULL, NULL);
			if (!g_file_set_contents (fichier_config, chemin, -1, &error))
			{
				if (error->message)
					printf ("%s\n", error->message);
				gtk_main_quit ();
			}
			g_free (chemin);
		}
	}
  g_free (fichier_config);
  g_free (chemin_soft);

  // Parser de la ligne de commande
  context = g_option_context_new (" - chess board game");
  g_option_context_add_main_entries (context, entries, NULL);
  g_option_context_add_group (context, gtk_get_option_group (TRUE));
  g_option_context_parse (context, &argc, &argv, &error);
  g_option_context_free (context);

  return TRUE;
}

// fonction d'initialisation principale du jeu
GList *echec_init (GList * liste_coups_jouables)
{
  register gint x, y;
	gint i;
  gchar *label;

  // mise à -1 du tableau echiquier
  for (x = 0; x < 144; x++)
    {
      echiquier[x].type_piece = -1;
      echiquier[x].ponderation = 1;
    }

  // mise à 0 des cases utilisables pour l'échiquier
  for (x = 0; x < 8; x++)
    {
      echiquier[x + 26].type_piece = vide;
      echiquier[x + 38].type_piece = vide;
      echiquier[x + 50].type_piece = vide;
      echiquier[x + 62].type_piece = vide;
      echiquier[x + 74].type_piece = vide;
      echiquier[x + 86].type_piece = vide;
      echiquier[x + 98].type_piece = vide;
      echiquier[x + 110].type_piece = vide;
    }

  // initialisation des coordonnées x & y du tableau echiquier
  for (x = 1; x < 9; x++)
    {
      echiquier[x + 25].x = x;
      echiquier[x + 25].y = 8;
      echiquier[x + 37].x = x;
      echiquier[x + 37].y = 7;
      echiquier[x + 49].x = x;
      echiquier[x + 49].y = 6;
      echiquier[x + 61].x = x;
      echiquier[x + 61].y = 5;
      echiquier[x + 73].x = x;
      echiquier[x + 73].y = 4;
      echiquier[x + 85].x = x;
      echiquier[x + 85].y = 3;
      echiquier[x + 97].x = x;
      echiquier[x + 97].y = 2;
      echiquier[x + 109].x = x;
      echiquier[x + 109].y = 1;
    }

  // Initialisation de la pondération des cases de l'échiquier
  for (x = 1; x < 7; x++)
    {
      echiquier[x + 38].ponderation = 2;
      echiquier[x + 50].ponderation = 2;
      echiquier[x + 62].ponderation = 2;
      echiquier[x + 74].ponderation = 2;
      echiquier[x + 86].ponderation = 2;
      echiquier[x + 98].ponderation = 2;
    }
  for (x = 0; x < 4; x++)
    {
      echiquier[x + 52].ponderation = 5;
      echiquier[x + 64].ponderation = 5;
      echiquier[x + 76].ponderation = 5;
      echiquier[x + 88].ponderation = 5;
    }
  for (x = 0; x < 2; x++)
    {
      echiquier[x + 65].ponderation = 10;
      echiquier[x + 77].ponderation = 10;
    }

  //initialisation de table_piece

  // initialisation des pions blancs et noirs
  for (x = 1; x < 9; x++)
    {
      table_piece[x] = initialisation_piece (x, pion, blanche, 97 + x, 1);
      table_piece[x + 16] =
	initialisation_piece (x + 16, pion, noire, 37 + x, 1);
    }

  // initialisation des autres pieces blanches
  table_piece[9] = initialisation_piece (9, tour, blanche, 110, 5);
  table_piece[10] = initialisation_piece (10, tour, blanche, 117, 5);
  table_piece[11] = initialisation_piece (11, cavalier, blanche, 111, 3);
  table_piece[12] = initialisation_piece (12, cavalier, blanche, 116, 3);
  table_piece[13] = initialisation_piece (13, fou, blanche, 112, 3);
  table_piece[14] = initialisation_piece (14, fou, blanche, 115, 3);
  table_piece[15] = initialisation_piece (15, dame, blanche, 113, 9);
  table_piece[16] = initialisation_piece (16, roi, blanche, 114, 200);

  // initialisation des autres pieces noires
  table_piece[25] = initialisation_piece (25, tour, noire, 26, 5);
  table_piece[26] = initialisation_piece (26, tour, noire, 33, 5);
  table_piece[27] = initialisation_piece (27, cavalier, noire, 27, 3);
  table_piece[28] = initialisation_piece (28, cavalier, noire, 32, 3);
  table_piece[29] = initialisation_piece (29, fou, noire, 28, 3);
  table_piece[30] = initialisation_piece (30, fou, noire, 31, 3);
  table_piece[31] = initialisation_piece (31, dame, noire, 29, 9);
  table_piece[32] = initialisation_piece (32, roi, noire, 30, 200);

  // mise en place des pieces sur l'échiquier en utilisant la table_piece

  for (x = 1; x < 33; x++)
    {
      echiquier[table_piece[x].place].type_piece = table_piece[x].type;
      echiquier[table_piece[x].place].couleur_piece = table_piece[x].couleur;
    }

  // Initialisation de la variable "couleur_case" de la table echiquier
  y = 0;
  do
    {
      x = 0;
      do
	{
	  echiquier[(y + 26) + x].couleur_case = blanche;
	  echiquier[(y + 27) + x].couleur_case = noire;
	  echiquier[(y + 39) + x].couleur_case = blanche;
	  echiquier[(y + 38) + x].couleur_case = noire;
	  x = x + 2;
	}
      while (x < 8);
      y = y + 24;
    }
  while (y < 96);

  // calcul des coups jouables pour chaque pièce
  liste_coups_jouables = calcul_cases_jouables (liste_coups_jouables);

  // initialisation de la couleur jouable sur l'échiquier
  humain_joue = blanc;

  // Couleur joué par l'humain
  couleur_humain = blanc;

  // Gnuchess ne joue pas contre l'humain lors de l'initialisation du jeu
  gnuchess = FALSE;

  // Libération mémoire du tableau de sauvegarde de la partie en cours au cas où
  if (num_coup_joue != 0)
    for (i = 0; i < num_coup_joue; i++)
      g_free (tab_sauve_partie[i].coup);

  // Initialisation du nombre de coup joués
  num_coup_joue = 0;

  // Sauvegarde de la position initiale de l'échiquier
  label = g_strdup ("Linechec");	// Libéré par la fonction sauvegarde_position
  sauvegarde_position (label);

  // Incrémentation du nombre de coups joués
  num_coup_joue++;

  // Initialisation de l'état du jeu
  partie_en_cours = FALSE;
  partie_enregistree = FALSE;

  return liste_coups_jouables;
}
